###########################################################
####################Macros and Functions###################
library(MLEcens)
library(survival)


###raw data generation function
raw_generator=function(med.surv,n,time.scan)
{
    rate = log(2)/med.surv
    pfs=rexp(n,rate)
    pfs[pfs==0]=min(pfs[pfs!=0])
    time.scan = sort(time.scan)
    if(max(pfs) > max(time.scan)){
        inter.temp = time.scan[length(time.scan)]-time.scan[length(time.scan)-1]
        time.scan = c(time.scan, seq(from = max(time.scan)+inter.temp, to = max(pfs)+inter.temp,by=inter.temp))
    }
    
    pfs.temp = matrix(rep(pfs,length(time.scan)),nrow=length(time.scan),byrow=T)
    pfs.temp = pfs.temp - time.scan
    pfs.disc = apply(pfs.temp, 2, function(v_pfs) which(v_pfs<=0)[1])
    pfs.disc
}



###observation generation function
#p1,p2,p3can not be 0, use very small value instead
ob_generator=function(raw,p1,p2,p3,std_time)  
{
    #std_time=length(time.scan)
    n=length(raw)
    p=matrix(0,n,max(raw)+5)
    ob=rep(0,n)
    for (i in 1:n)
    {
        p1k=0
        if (raw[i]>1)
            p1k=p1^seq(from=raw[i]-1, to=1,by=-1)
        p3k=p3^seq(from=1,to=max(raw)+5-raw[i],by=1)
        c=(1-p2)/(sum(p1k)+sum(p3k))
        temp=c(c*p1k,p2,c*p3k)
        p[i,]=temp[temp>0]
        ob[i]=sample(1:(max(raw)+5),1,prob=p[i,])
    }
    ob[ob>std_time]=std_time+0.5        	#administrative censoring
    ob
}


###informative censoring function (extra imaging included)
inf_cen=function(le,cr0,int_extr)
{
    dif=cr0-le
    cr0[dif>=(int_extr+0.5)]=le[dif>=(int_extr+0.5)]+int_extr+0.5
    cr0
}


###independent censoring function
ind_cen=function(le,cr,cen.rate.com,cen.rate.cr1, cen.rate.cr2)
{
    n=length(cr)
    
    time.min.temp = apply(cbind(le, cr),1,min)
    id.subset = setdiff(1:n, which(time.min.temp==1))
    n.subset=length(id.subset)
    
    if (round(n.subset*cen.rate.com)>=1)
    {        
        id.cen = sample(id.subset, size=round(n.subset*cen.rate.com))
        time.end = apply(cbind(le, cr),1,min)[id.cen]
        time.cen = sapply(time.end,function(time) sample(1:time,size=1,prob=1:time))
        
        ind=(le[id.cen]>=time.cen)
        le[id.cen][ind]=time.cen[ind]-0.5
        ind=(cr[id.cen]>=time.cen)
        cr[id.cen][ind]=time.cen[ind]-0.5        
    }
        
    if (round(n.subset*cen.rate.cr2)>=1)
    {        
        id.cen2 = sample(id.cen, size=round(n.subset*cen.rate.cr2))
        time.end = cr[id.cen2]
        time.cen = sapply(time.end,function(time) sample(1:time,size=1,prob=1:time))
        
        ind=(cr[id.cen2]>=time.cen)
        cr[id.cen2][ind]=time.cen[ind]-0.5        
    }
    
    
    if (round(n.subset*cen.rate.cr1)>=1)
    {        
        id.rest = setdiff(id.subset,id.cen)
        id.cen = sample(id.rest, size=round(n.subset*cen.rate.cr1))
        time.end = cr[id.cen]
        time.cen = sapply(time.end,function(time) sample(1:time,size=1,prob=1:time))

        ind=(cr[id.cen]>=time.cen)
        cr[id.cen][ind]=time.cen[ind]-0.5        
    }
        
    data=cbind(le,cr)
    data        
}


###representing paired data as rectangle
rec=function(le,cr,std_time)
{
    #std_time=length(time.scan)
    x1=ceiling(le)-1
    x2=le
    x2[le%%1==0.5]=std_time+1
    y1=ceiling(cr)-1
    y2=cr
    y2[cr%%1==0.5]=std_time+1
    data=cbind(x1,x2,y1,y2)
    data
}


###redistributed contingency table (equal risk)
recont=function(mle,std_time)
{
    #std_time=length(time.scan)
    maxint=mle$rects
    recp=mle$p
    cont=matrix(0,std_time+1,std_time+1)
    n=length(recp)
    for (i in 1:n)
    {
        margin=maxint[i,]
        x1=margin[1]
        x2=margin[2]
        y1=margin[3]
        y2=margin[4]    	
        x=matrix(replicate(y2-y1,(x1+1):x2),nrow=(x2-x1),ncol=(y2-y1))
        y=matrix(t(replicate(x2-x1,(y1+1):y2)),nrow=(x2-x1),ncol=(y2-y1))
        temp=abs(x-y)				#|i-j|
        
        #equal risk (opposite proportitional to weight), +0.5 to deal with i=j
        cont[(x1+1):x2,(y1+1):y2]=recp[i]*(1/(temp+0.5))/sum(1/(temp+0.5)) 
    }
    cont
}


###differential risk ratio estimation
dr=function(cont)
{

    n=dim(cont)[1]   
    x=replicate(n,1:n)
    y=t(replicate(n,1:n))
    weight=(y-x)/(n-1)    	#|i-j|/|t-1|,but since we will subtract later, simply use j-i.
    dr=sum(weight*cont)    
    dr
    
}



###EDR
edr=function(le,cr)
{
    denom=sum(le%%1==0)
    b=sum(le%%1==0&cr%%1==0.5)
    temp=(le%%1==0&cr%%1==0)
    a3=sum(le[temp]<cr[temp])
    edr_val=(b+a3)/denom
    edr_val	
}


###LDR
ldr=function(le,cr)
{
    denom=sum(le!=cr)
    c=sum(le%%1==0.5&cr%%1==0)
    temp=(le%%1==0&cr%%1==0)
    a2=sum(le[temp]>cr[temp])
    ldr_val=(c+a2)/denom
    ldr_val
}


###hazard ratio estimation
hre=function(con,exp, time.scan)
{
    n_con=length(con)
    n_exp=length(exp)
    
    con_event=rep(0,length=n_con)
    ind=(con%%1==0)
    con_event[ind]=1
    con_ind=rep(0,n_con)
    con_time = c(0,time.scan)[con+1]
    con_data=cbind(con_time,con_event,con_ind)
    
    exp_event=rep(0,n_exp)
    ind=(exp%%1==0)
    exp_event[ind]=1
    exp_ind=rep(1,n_exp)
    exp_time = c(0,time.scan)[exp+1]
    exp_data=cbind(exp_time,exp_event,exp_ind)
    
    data_all=rbind(con_data,exp_data)
    data_all=data.frame(time=data_all[,1],event=data_all[,2],arm=data_all[,3])
    cox=coxph(Surv(time,event)~arm,data=data_all)
    hr_value=summary(cox)$coefficients[2]
    hr_value
    
}

hre_com = function(con.time, con.event, exp.time, exp.event){
    con_data=data.frame(time=con.time, event=con.event, trt=0)
    exp_data=data.frame(time=exp.time, event=exp.event, trt=1)
    data_all=rbind(con_data, exp_data)
    cox=coxph(Surv(time,event)~trt,data=data_all)
    hr_value=summary(cox)$coefficients[2]
    hr_value
    
}



###variance of difference of index estimation by nonparametric bootstrapping
std_boot=function(con_rec_sample,exp_rec_sample,std_time,boot_num)
{
    
    con_dr_boot=rep(0,boot_num)
    exp_dr_boot=rep(0,boot_num)
    ddr_boot=rep(0,boot_num)
    for (i in 1:boot_num)
    {
        con_rec_bootid=sample(1:dim(con_rec_sample)[1],size=dim(con_rec_sample)[1],replace=TRUE)
        exp_rec_bootid=sample(1:dim(exp_rec_sample)[1],size=dim(exp_rec_sample)[1],replace=TRUE)
        con_rec_boot=con_rec_sample[con_rec_bootid,]
        exp_rec_boot=exp_rec_sample[exp_rec_bootid,]
        
        con_mle_boot=computeMLE(con_rec_boot,B=c(1,0),max.inner=100,max.outer=1800)
        con_cont_boot=recont(con_mle_boot,std_time)
        exp_mle_boot=computeMLE(exp_rec_boot,B=c(1,0),max.inner=100,max.outer=1800)
        exp_cont_boot=recont(exp_mle_boot,std_time)
        
        con_dr_boot[i]=dr(con_cont_boot)
        exp_dr_boot[i]=dr(exp_cont_boot)
        ddr_boot[i]=exp_dr_boot[i]-con_dr_boot[i]
        
    }
    
    std_con_dr_boot=sd(con_dr_boot)
    std_exp_dr_boot=sd(exp_dr_boot)
    std_ddr_boot=sd(ddr_boot)
    std_ddr_boot
    
}


###finding the critical region
c_value=function(alpha,ddr_range,std_ddr)
{
    z_alpha=qnorm(alpha)
    ite_num=floor(-1*z_alpha/0.01)
    ite_alpha=rep(0,ite_num)
    for (k in 1:ite_num)
    {
        critic=(-1*z_alpha-0.01*k)*std_ddr
        ite_alpha[k]=pnorm(-1*critic/std_ddr)-pnorm((-2*ddr_range+critic)/std_ddr)
    }
    critic_ind=which.min(abs(ite_alpha-alpha))
    critic=(-1*z_alpha-0.01*critic_ind)*std_ddr
    critic	
}


##### transforming the survival data to working data ####
tran.surv = function(time,event,time.scan){
    time.temp = matrix(rep(time,length(time.scan)),nrow=length(time.scan),byrow=T)
    time.temp = time.temp - time.scan
    time.disc = apply(time.temp, 2, function(v_pfs) which.min(abs(v_pfs))[1])
    time.disc[event==0]=time.disc[event==0]+0.5
    time.disc    
}




ddr_proc = function(con.time.le, con.event.le, con.time.cr, con.event.cr, 
                     exp.time.le, exp.event.le, exp.time.cr, exp.event.cr, 
                     time.scan, int_extr, simu_num=100, 
                     boot_num=8000, alpha=0.05, power=0.8, HRR_dif=0.2){
    #le: local; cr: central review
    #con.time: vector, survival time of control arm
    #con.event: vector, event indicator of con arm, 1: event, 0 censor.
    #exp.time: vector, survival time of exp arm
    #exp.event: vector: event indicator of exp arm, 1: event, 0 censor.
    #time.scan: vector, scan time, in the same unit as survival time
    #int_extr: extra scans for BICR after LE progression
    #simu_num: #number of simulations to calculate slope of HRR vs Ddr
    #boot_num: # of bootstrap to calculate se of ddr estimator
    #HRR_dif: acceptable difference of HRR from 1
    
    data1 = data.frame(con.time.le=con.time.le, con.event.le=con.event.le, con.time.cr=con.time.cr, con.event.cr=con.event.cr)
    data2 = data.frame(exp.time.le=exp.time.le, exp.event.le=exp.event.le, exp.time.cr=exp.time.cr, exp.event.cr=exp.event.cr)
    if(!all(complete.cases(data1))||!all(complete.cases(data2))) stop("Please make sure no missing values (NA).")
    
    simu_par = try(read.csv("simu_par.csv", header=T, stringsAsFactors=F), silent=T)
    if(class(simu_par) == "try-error") stop("Please put simu_par.txt in the current working directory.")
    
    con.med = summary(survfit(Surv(c(con.time.le,con.time.cr), c(con.event.le,con.event.cr))~1))$table["median"]
    exp.med = summary(survfit(Surv(c(exp.time.le,exp.time.cr), c(exp.event.le,exp.event.cr))~1))$table["median"]
    con_n = length(con.time.le)  #control num of patients
    exp_n = length(exp.time.le)  #experim num of patients
    std_time = length(time.scan) #study end time
    
        
    con_dr_all=NULL
    con_edr_all=NULL
    con_ldr_all=NULL
    
    exp_dr_all=NULL
    exp_edr_all=NULL
    exp_ldr_all=NULL
    
    le_hr_all=NULL
    cr_hr_all=NULL
    hrr_all=NULL
    
    
    ###### derive slope of HRR vs Ddr
    for (k in 1:dim(simu_par)[1]){
        con_p1_le=simu_par[k,"con_p1_le"]				#par to generate early prob for LE
        con_p2_le=simu_par[k,"con_p2_le"]				#agreement prob for LE
        con_p3_le=simu_par[k,"con_p3_le"]				#par to generate late prob for LE
        con_p1_cr=simu_par[k,"con_p1_cr"]				#par to generate early prob for CR
        con_p2_cr=simu_par[k,"con_p2_cr"]			  #agreement prob for CR
        con_p3_cr=simu_par[k,"con_p3_cr"]				#par to generate late prob for CR
        
        exp_p1_le=simu_par[k,"exp_p1_le"]				#par to generate early prob for LE
        exp_p2_le=simu_par[k,"exp_p2_le"]			#agreement prob for LE
        exp_p3_le=simu_par[k,"exp_p3_le"]			#par to generate late prob for LE
        exp_p1_cr=simu_par[k,"exp_p1_cr"]			#par to generate early prob for CR
        exp_p2_cr=simu_par[k,"exp_p2_cr"]			#agreement prob for CR
        exp_p3_cr=simu_par[k,"exp_p3_cr"]			#par to generate late prob for CR
        case_num=simu_par[k,"case_num"]    		#index for different cases (plotting purpose)
        
        #raw data generation
        if (case_num==1){
            con_raw=raw_generator(con.med,con_n*1000/min(con_n,exp_n),time.scan)
            exp_raw=raw_generator(exp.med,exp_n*1000/min(con_n,exp_n),time.scan)
        }
        
        #multiple simulations to see the variance of estimator
        #variation comes from the uncertainty of the obs from LE and BICR and censoring
        con_dr=rep(0,simu_num)
        con_edr=rep(0,simu_num)
        con_ldr=rep(0,simu_num)
        exp_dr=rep(0,simu_num)
        exp_edr=rep(0,simu_num)
        exp_ldr=rep(0,simu_num)
        le_hr=rep(0,simu_num)
        cr_hr=rep(0,simu_num)
        hrr=rep(0,simu_num)
        
        
        for (j in 1:simu_num){
            
            #observation generation
            con_le=ob_generator(con_raw,con_p1_le,con_p2_le,con_p3_le,std_time)
            con_cr0=ob_generator(con_raw,con_p1_cr,con_p2_cr,con_p3_cr,std_time)  #CR ob without info censoring
            con_cr=inf_cen(con_le,con_cr0,int_extr)						#CR ob after info censoring
            con_data=cbind(con_le, con_cr)
            con_le=con_data[,1]
            con_cr=con_data[,2]
            
            exp_le=ob_generator(exp_raw,exp_p1_le,exp_p2_le,exp_p3_le,std_time)
            exp_cr0=ob_generator(exp_raw,exp_p1_cr,exp_p2_cr,exp_p3_cr,std_time)	#CR ob without info censoring
            exp_cr=inf_cen(exp_le,exp_cr0,int_extr)						#CR ob after info censoring
            exp_data=cbind(exp_le,exp_cr)
            exp_le=exp_data[,1]
            exp_cr=exp_data[,2]
            
            #representing data as rectangle
            con_rec=rec(con_le,con_cr,std_time)
            exp_rec=rec(exp_le,exp_cr,std_time)
            
            
            #estimate the mle for rec and redistribute prob for blocks covering >1 cell with equal risk
            #computeMLE is a function in the package "MLEcens"
            con_mle=computeMLE(con_rec,B=c(1,0),max.inner=100,max.outer=1800)
            con_cont=recont(con_mle,std_time)
            exp_mle=computeMLE(exp_rec,B=c(1,0),max.inner=100,max.outer=1800)
            exp_cont=recont(exp_mle,std_time)
            
            #Differential Risk Ratio estimation
            con_dr[j]=dr(con_cont)
            exp_dr[j]=dr(exp_cont)
            
            #EDR estimation
            con_edr[j]=edr(con_le,con_cr)
            exp_edr[j]=edr(exp_le,exp_cr)	
            
            #LDR estimation
            con_ldr[j]=ldr(con_le,con_cr)
            exp_ldr[j]=ldr(exp_le,exp_cr)
            
            #HR estimation
            le_hr[j]=hre(con_le,exp_le, time.scan)
            cr_hr[j]=hre(con_cr,exp_cr, time.scan)
            hrr[j]=cr_hr[j]/le_hr[j]
        }
        
        
        con_dr_all=cbind(con_dr_all,con_dr)
        con_edr_all=cbind(con_edr_all,con_edr)
        con_ldr_all=cbind(con_ldr_all,con_ldr)
        
        exp_dr_all=cbind(exp_dr_all,exp_dr)
        exp_edr_all=cbind(exp_edr_all,exp_edr)
        exp_ldr_all=cbind(exp_ldr_all,exp_ldr)
        
        le_hr_all=cbind(le_hr_all,le_hr)
        cr_hr_all=cbind(cr_hr_all,cr_hr)
        hrr_all=cbind(hrr_all,hrr)              
        
    }
    
    hrr_all_mean=colMeans(hrr_all)
    ddr_all=exp_dr_all-con_dr_all
    ddr_all_mean=colMeans(ddr_all)

    slope = summary(lm(hrr_all_mean~ddr_all_mean))$coefficients["ddr_all_mean","Estimate"]
    r.squared = summary(lm(hrr_all_mean~ddr_all_mean))$r.squared
    lm.sigma = summary(lm(hrr_all_mean~ddr_all_mean))$sigma

    simu.res = list(slope=slope, r.squared=r.squared, lm.sigma=lm.sigma, hrr_all_mean=hrr_all_mean, ddr_all_mean=ddr_all_mean, 
                    ddr_all=ddr_all, hrr_all=hrr_all, con_dr_all=con_dr_all, con_edr_all=con_edr_all, 
                    con_ldr_all=con_ldr_all, exp_dr_all=exp_dr_all, exp_edr_all=exp_edr_all, exp_ldr_all=exp_ldr_all, 
                    le_hr_all=le_hr_all, cr_hr_all=cr_hr_all)
    
    
    ######### testing #######
    ddr_range=HRR_dif/abs(slope)
    
    con.time.le.disc = tran.surv(con.time.le,con.event.le,time.scan)
    con.time.cr.disc = tran.surv(con.time.cr,con.event.cr,time.scan)
    exp.time.le.disc = tran.surv(exp.time.le,exp.event.le,time.scan)
    exp.time.cr.disc = tran.surv(exp.time.cr,exp.event.cr,time.scan)
    
    #representing data as rectangle
    con_rec=rec(con.time.le.disc,con.time.cr.disc,std_time)
    exp_rec=rec(exp.time.le.disc,exp.time.cr.disc,std_time)
    
    #computeMLE is a function in the package "MLEcens"
    con_mle=computeMLE(con_rec,B=c(1,0),max.inner=100,max.outer=1800)
    con_cont=recont(con_mle,std_time)
    exp_mle=computeMLE(exp_rec,B=c(1,0),max.inner=100,max.outer=1800)
    exp_cont=recont(exp_mle,std_time)
    
    
    #Differential Risk Ratio estimation
    con_dr=dr(con_cont)
    exp_dr=dr(exp_cont)
    
    #EDR estimation
    con_edr=edr(con.time.le.disc,con.time.cr.disc)
    exp_edr=edr(exp.time.le.disc,exp.time.cr.disc)    
    
    #LDR estimation
    con_ldr=ldr(con.time.le.disc,con.time.cr.disc)
    exp_ldr=ldr(exp.time.le.disc,exp.time.cr.disc)
    
    #HR estimation
    le_hr = hre_com(con.time.le, con.event.le, exp.time.le, exp.event.le)
    cr_hr = hre_com(con.time.cr, con.event.cr, exp.time.cr, exp.event.cr)
    hrr=cr_hr/le_hr
    
    
    #estimating the std of Ddr of the selected sample
    std_ddr=std_boot(con_rec,exp_rec,std_time,boot_num)
    
    #get the theoretical critical region
    critic_value=c_value(alpha,ddr_range,std_ddr)    
    critic_region=c(critic_value-ddr_range,ddr_range-critic_value)
    
    ddr=exp_dr-con_dr
    reject.hypo = (ddr>=critic_region[1])&(ddr<=critic_region[2])
    
    test.res = list(reject.hypo=reject.hypo, ddr=ddr, std_ddr=std_ddr, ddr_range=ddr_range,
                    critic_region=critic_region, critic_value=critic_value, hrr=hrr, 
                    le_hr=le_hr, cr_hr=cr_hr, con_dr=con_dr, exp_dr=exp_dr, con_edr=con_edr, 
                    exp_edr=exp_edr, con_ldr=con_ldr,exp_ldr=exp_ldr)
    
        
    ###### calculating required sample size ##########
    ite_num=6/0.01
    ite_alpha=rep(0,ite_num)
    for (k in 1:ite_num)
    {
        zval1=qnorm(1-(1-power)/2)-k*0.01
        zval2=-1*qnorm(1-(1-power)/2)-k*0.01
        ite_alpha[k]=pnorm(zval1)-pnorm(zval2)
    }
    temp_ind=which.min(abs(ite_alpha-alpha))[1]
    req_std=ddr_range/(temp_ind*0.01)
    req_n=ceiling(c(con_n,exp_n)*(std_ddr/req_std)^2)
    req_n=data.frame(con_n=req_n[1], exp_n=req_n[2])
    
    list(reject.hypo=test.res$reject.hypo, ddr=test.res$ddr, std_ddr=test.res$std_ddr, 
         ddr_range=test.res$ddr_range, critic_region=test.res$critic_region, 
         critic_value=test.res$critic_value, hrr=test.res$hrr, req_n=req_n,
         slope=simu.res$slope, r.squared=simu.res$r.squared, lm.sigma=simu.res$lm.sigma, 
         hrr_all_mean=simu.res$hrr_all_mean, ddr_all_mean=simu.res$ddr_all_mean,
         simu.res=simu.res, test.res=test.res)
}


